#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

# アプリケーションバージョン取得用のパターンファイル

module ApplicationPatterns

  # name: アプリケーション名
  # find_patterns: 大まかにファイルを抽出するパターン
  #   * 全アプリケーションのfindパターンを結合したパターンをfindに設定し、
  #     一度のfind実行で済むようにする。
  #   * findコマンドで抽出したリストを、再度パターンで検索したものをcheck_proc
  #     に渡す。
  # check_proc: 対象パスのファイルを解析してバージョンを抽出する。
  APPLICATIONS = {
    :Struts => {
      :update => '2018/04/06',
      :name => 'Apache Struts',
      :pids => [ 516 ],
      :find_patterns =>
      [ { :pattern => '.*/struts2-.*\.jar$', :type => 'f' },
        { :pattern => '.*/struts-.*\.jar$', :type => 'f' } ],
      :check_proc => proc {|obj, path|
        ret = { :success => false }
        if FileTest.readable?(path) then
          zipout = obj.extract_zip(path, 'META-INF/MANIFEST.MF')
          if zipout =~ /^(Bundle-Name|Specification-Title): (Struts 2 Core|Struts Core|Struts Framework)/ and
              zipout =~ /^(Implementation-Version|Bundle-Version): (\d+\.\d+(\.\d+)?)/ then
            ret[:application] = 'Apache Struts'
            ret[:version] = Regexp.last_match(2)
            ret[:success] = true
          end
        end
        ret
      },
    },
    :Tomcat => {
      :update => '2018/04/06',
      :name => 'Apache Tomcat',
      :pids => [ 93 ],
      :find_patterns =>
      [ { :pattern => '.*/catalina\.jar$', :type => 'f' } ],
      :check_proc => proc {|obj, path|
        ret = { :success => false }
        if FileTest.readable?(path) then
          zipout = obj.extract_zip(path, 'META-INF/MANIFEST.MF')
          if zipout =~ /^(Implementation|Specification)-Title: Apache Tomcat/ and
              zipout =~ /^Implementation-Version: (\d+\.\d+(\.\d+)?)/ then
            ret[:application] = 'Apache Tomcat'
            ret[:version] = Regexp.last_match(1)
            ret[:success] = true
          end
        end
        ret
      },
    },
    :PHP => {
      :update => '2018/04/06',
      :name => 'PHP',
      :pids => [ 94 ],
      :find_patterns =>
      [ { :pattern => '.*/php[0-9.]*', :type => 'f' },
        { :pattern => '.*/libphp[0-9.]*\.so', :type => 'f' },
        { :pattern => '.*/mod_php[0-9.]*\.so', :type => 'f' } ],
      :check_proc => proc {|obj, path|
        ret = { :success => false }
        if FileTest.readable?(path) then
          bin = ''
          File.open(path, 'r') do |f|
            bin = f.read
          end
          bin = obj.to_ascii8bit(bin)
          if bin =~ /X-Powered-By: PHP\/(\d+\.\d+\.\d+)/ then
            ret[:application] = 'PHP'
            ret[:version] = Regexp.last_match(1)
            ret[:success] = true
          end
        end
        ret
      },
    },
    :Ruby => {
      :update => '2018/04/06',
      :name => 'Ruby',
      :pids => [ 95 ],
      :find_patterns => [ { :pattern => '.*/ruby[0-9]*$', :type => 'f' } ],
       :check_proc => proc {|obj, path|
        ret = { :success => false }
        if FileTest.executable?(path) then
          cmdout = obj.exec_command_stdout("#{path} --version")
          if /ruby (\d+\.\d+\.\d+(p\d+)?) /m =~ cmdout then
            ret[:application] = 'ruby'
            ret[:version] = Regexp.last_match(1)
            ret[:success] = true
          end
        end
        ret
      },
    },
   :'Oracle Java SE JDK' => {
      :update => '2018/05/21',
      :name => 'Oracle Java SE JDK',
      :pids => [ 204 ],
      :find_patterns => [ { :pattern => '.*/bin/java$', :type => 'f' } ],
       :check_proc => proc {|obj, path|
        ret = { :success => false }
        if FileTest.executable?(path) then
          cmderr = obj.exec_command_stderr("#{path} -version")
          if /java version "(\d+)"/m =~ cmderr then
            ### FIXME: JDK判定処理を追加
            ret[:application] = 'Oracle Java'
            ret[:version] = Regexp.last_match(1)
            ret[:success] = true
          end
        end
        ret
      },
    },
   :'Oracle Java SE JRE' => {
      :update => '2018/05/21',
      :name => 'Oracle Java SE JRE',
      :pids => [ 202 ],
      :find_patterns => [ { :pattern => '.*/bin/java$', :type => 'f' } ],
       :check_proc => proc {|obj, path|
        ret = { :success => false }
        if FileTest.executable?(path) then
          cmderr = obj.exec_command_stderr("#{path} -version")
          if /java version "(\d+)"/m =~ cmderr then
            ### FIXME: JRE判定処理を追加
            ret[:application] = 'Oracle Java'
            ret[:version] = Regexp.last_match(1)
            ret[:success] = true
          end
        end
        ret
      },
    },
  }

end
